package com.ruoyi.system.service.impl;

import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.TPatientHospitalDoctor;
import com.ruoyi.system.domain.TRecord;
import com.ruoyi.system.domain.req.TRecordResultReq;
import com.ruoyi.system.domain.resp.RecordHistoryDataResp;
import com.ruoyi.system.domain.resp.TRecordResp;
import com.ruoyi.system.mapper.TRecordMapper;
import com.ruoyi.system.service.ITRecordService;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 诊断记录信息Service业务层处理
 *
 * @author ruoyi
 * @date 2022-08-03
 */
@Service
public class TRecordServiceImpl implements ITRecordService
{
    private static final Logger logger = LoggerFactory.getLogger(TRecordServiceImpl.class);

    @Autowired
    private TRecordMapper tRecordMapper;

    /**
     * 查询诊断记录信息
     *
     * @param id 诊断记录信息主键
     * @return 诊断记录信息
     */
    @Override
    public TRecord selectTRecordById(Long id)
    {
        return tRecordMapper.selectTRecordById(id);
    }

    /**
     * 查询诊断记录信息列表
     *
     * @param tRecord 诊断记录信息
     * @return 诊断记录信息
     */
    @Override
    public List<TRecord> selectTRecordList(TRecord tRecord)
    {
        return tRecordMapper.selectTRecordList(tRecord);
    }

    /**
     * 新增诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int insertTRecord(TRecord tRecord)
    {
        tRecord.setCreateTime(DateUtils.getNowDate());
        return tRecordMapper.insertTRecord(tRecord);
    }

    /**
     * 修改诊断记录信息
     *
     * @param tRecord 诊断记录信息
     * @return 结果
     */
    @Override
    public int updateTRecord(TRecord tRecord)
    {
        tRecord.setUpdateTime(DateUtils.getNowDate());
        if (null == tRecord.getAiResultModifyTime()) {
            tRecord.setAiResultModifyTime(DateUtils.getNowDate());
        }
        return tRecordMapper.updateTRecord(tRecord);
    }

    /**
     * 批量删除诊断记录信息
     *
     * @param ids 需要删除的诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordByIds(Long[] ids)
    {
        return tRecordMapper.deleteTRecordByIds(ids);
    }

    /**
     * 删除诊断记录信息信息
     *
     * @param id 诊断记录信息主键
     * @return 结果
     */
    @Override
    public int deleteTRecordById(Long id)
    {
        return tRecordMapper.deleteTRecordById(id);
    }

    @Override
    public List<TRecordResp> selectTRecordListWithoutResult(TRecordResultReq tRecord)
    {
        return tRecordMapper.selectTRecordListWithoutResult(tRecord);
    }

    @Override
    public List<TRecordResp> queryHomePatientList(TRecordResultReq tRecord) {
        List<TRecordResp> tRecordResps = tRecordMapper.selectHomePatientList(tRecord);
        return tRecordResps;
    }

    @Override
    public TRecordResp selectById(Long id)
    {
        return tRecordMapper.selectById(id);
    }

    @Override
    public List<TRecordResp> queryRecordListByPHDids(TPatientHospitalDoctor tPatientHospitalDoctor)
    {
        return tRecordMapper.queryRecordListByPHDids(tPatientHospitalDoctor);
    }

    @Override
    public AjaxResult getHistoryList(TRecord tRecord)
    {
        List<TRecord> tRecords = tRecordMapper.selectTRecordList(tRecord);
        tRecords.removeIf(o->"0".equals(o.getStatus()));
        if (CollectionUtils.isEmpty(tRecords)){
            return AjaxResult.success();
        }
        List<TRecord> records = tRecords.stream().sorted(Comparator.comparing(TRecord::getCreateTime)).collect(Collectors.toList());
        List<String> jsonKeyList = new ArrayList<String>(){{
            add("shemian-img_sz");
            add("img_st");
            add("img_sx");
        }};
        RecordHistoryDataResp recordHistoryDataRespMap = getDataList(records,jsonKeyList);
        return AjaxResult.success(recordHistoryDataRespMap);
    }

    private RecordHistoryDataResp getDataList(List<TRecord> records, List<String> jsonKeyList)
    {

        RecordHistoryDataResp recordHistoryDataResp = new RecordHistoryDataResp();
        List<Date> dateList = records.stream().map(TRecord::getCreateTime).collect(Collectors.toList());
        recordHistoryDataResp.setDateList(dateList);
        List<JSONObject> result = records.stream().map(tRecord -> {
            JSONObject jsonObject = new JSONObject();
            if(StringUtils.isEmpty(tRecord.getUpdateResult2())){
                jsonObject = JSONObject.parseObject(tRecord.getAiResult2());
            }else if(!StringUtils.isEmpty(tRecord.getUpdateResult2())){
                jsonObject = JSONObject.parseObject(tRecord.getUpdateResult2());
            }
            return jsonObject;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(result)){
            List<Integer> shezhiList = new LinkedList<>();
            List<Integer> shetaiList = new LinkedList<>();
            List<Integer> shexiaList = new LinkedList<>();
            String[] keys0 = jsonKeyList.get(0).split("-");
            String key1 = jsonKeyList.get(1);
            String key2 = jsonKeyList.get(2);
            result.forEach(jsonObject -> {
                Integer value1 ;
                Integer value2 ;
                Integer value3 ;
                try
                {
                    value1 =  jsonObject.getJSONObject("data").getJSONObject("shezhi").getJSONObject(keys0[0]).getJSONObject(keys0[1]).getInteger("anomaly");
                    value2 = jsonObject.getJSONObject("data").getJSONObject("shetai").getJSONObject(key1).getInteger("anomaly");
                    value3 = jsonObject.getJSONObject("data").getJSONObject("shexia").getJSONObject(key2).getInteger("anomaly");
                }catch (Exception e){
                    throw new ServiceException("JSON格式错误");
                }
                shezhiList.add(value1);
                shetaiList.add(value2);
                shexiaList.add(value3);
            });
            recordHistoryDataResp.setShezhiList(shezhiList);
            recordHistoryDataResp.setShetaiList(shetaiList);
            recordHistoryDataResp.setShexiaList(shexiaList);
        }
        return recordHistoryDataResp;
    }

    /**
     * 查询医生诊断次数
     *
     * @param doctorId 医生id
     * @return int
     */
    @Override
    public int queryDoctorNumberOfDiagnosis(Long doctorId) {
        return tRecordMapper.queryDoctorNumberOfDiagnosis(doctorId);
    }
}
